# Standard Library
from datetime import datetime
from http import HTTPStatus
from unittest.mock import MagicMock, patch

# Django
from django.contrib.auth.models import Permission, User
from django.test import RequestFactory, TestCase
from django.urls import reverse

# AA SRP
from aasrp.models import SrpLink, SrpRequest
from aasrp.tests.utils import get_permission_content_type
from aasrp.views.ajax import srp_request_remove


class TestSrpRequestRemove(TestCase):
    """
    Test the srp_request_remove view.
    """

    @patch("aasrp.views.ajax.SrpRequest.objects.get")
    def test_srp_request_remove_success_for_user_with_perm_manage_srp(self, mock_get):
        """
        Test the srp_request_remove view for a user with the manage_srp permissions.

        :param mock_get:
        :type mock_get:
        :return:
        :rtype:
        """

        permission_content_type = get_permission_content_type()

        user = User.objects.create(username="testuser")
        permission = Permission.objects.get(
            content_type=permission_content_type, codename="manage_srp"
        )
        user.user_permissions.add(permission)
        self.client.force_login(user)
        srp_link = SrpLink.objects.create(srp_code="SRP123", fleet_time=datetime.now())
        srp_request = SrpRequest.objects.create(
            request_code="REQ123", srp_link=srp_link
        )
        srp_request.delete = MagicMock()
        mock_get.return_value = srp_request

        factory = RequestFactory()
        request = factory.get(
            reverse("aasrp:ajax_srp_request_remove", args=["SRP123", "REQ123"])
        )
        request.user = user

        response = srp_request_remove(request, "SRP123", "REQ123")

        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertJSONEqual(
            response.content,
            {"success": True, "message": "SRP request has been removed"},
        )
        srp_request.delete.assert_called_once()

    @patch("aasrp.views.ajax.SrpRequest.objects.get")
    def test_srp_request_remove_success_for_user_with_perm_manage_srp_requests(
        self, mock_get
    ):
        """
        Test the srp_request_remove view for a user with the manage_srp_requests permissions.

        :param mock_get:
        :type mock_get:
        :return:
        :rtype:
        """

        permission_content_type = get_permission_content_type()

        user = User.objects.create(username="testuser")
        permission = Permission.objects.get(
            content_type=permission_content_type, codename="manage_srp_requests"
        )
        user.user_permissions.add(permission)
        self.client.force_login(user)
        srp_link = SrpLink.objects.create(srp_code="SRP123", fleet_time=datetime.now())
        srp_request = SrpRequest.objects.create(
            request_code="REQ123", srp_link=srp_link
        )
        srp_request.delete = MagicMock()
        mock_get.return_value = srp_request

        factory = RequestFactory()
        request = factory.get(
            reverse("aasrp:ajax_srp_request_remove", args=["SRP123", "REQ123"])
        )
        request.user = user

        response = srp_request_remove(request, "SRP123", "REQ123")

        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertJSONEqual(
            response.content,
            {"success": True, "message": "SRP request has been removed"},
        )
        srp_request.delete.assert_called_once()

    @patch("aasrp.views.ajax.SrpRequest.objects.get")
    def test_srp_request_remove_not_found_for_user_with_perm_manage_srp(self, mock_get):
        """
        Test the srp_request_remove view for a user with the manage_srp permissions

        :param mock_get:
        :type mock_get:
        :return:
        :rtype:
        """

        permission_content_type = get_permission_content_type()

        user = User.objects.create(username="testuser")
        permission = Permission.objects.get(
            content_type=permission_content_type, codename="manage_srp"
        )
        user.user_permissions.add(permission)
        self.client.force_login(user)
        mock_get.side_effect = SrpRequest.DoesNotExist

        factory = RequestFactory()
        request = factory.get(
            reverse("aasrp:ajax_srp_request_remove", args=["SRP123", "REQ123"])
        )
        request.user = user

        response = srp_request_remove(request, "SRP123", "REQ123")

        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertJSONEqual(response.content, {"success": False})

    @patch("aasrp.views.ajax.SrpRequest.objects.get")
    def test_srp_request_remove_not_found_for_user_with_perm_manage_srp_requests(
        self, mock_get
    ):
        """
        Test the srp_request_remove view for a user with the manage_srp_requests permissions

        :param mock_get:
        :type mock_get:
        :return:
        :rtype:
        """

        permission_content_type = get_permission_content_type()

        user = User.objects.create(username="testuser")
        permission = Permission.objects.get(
            content_type=permission_content_type, codename="manage_srp_requests"
        )
        user.user_permissions.add(permission)
        self.client.force_login(user)
        mock_get.side_effect = SrpRequest.DoesNotExist

        factory = RequestFactory()
        request = factory.get(
            reverse("aasrp:ajax_srp_request_remove", args=["SRP123", "REQ123"])
        )
        request.user = user

        response = srp_request_remove(request, "SRP123", "REQ123")

        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertJSONEqual(response.content, {"success": False})



"""
Test the admin interface.
"""

# Standard Library
from http import HTTPStatus

# Django
from django.contrib.auth.models import User
from django.test import Client, TestCase
from django.urls import reverse

# AA SRP
from aasrp.models import FleetType


class TestFleetTypeAdmin(TestCase):
    """
    Test the FleetType admin interface.
    """

    @classmethod
    def setUpTestData(cls):
        """
        Set up test data for the admin interface tests.

        :return:
        :rtype:
        """

        cls.admin_user = User.objects.create_superuser(
            "admin", "admin@example.com", "password"
        )
        cls.client = Client()
        cls.client.login(username="admin", password="password")
        cls.fleet_type = FleetType.objects.create(name="Test Fleet", is_enabled=True)

    def test_admin_page_loads(self):
        """
        Test that the admin page loads correctly.

        :return:
        :rtype:
        """

        url = reverse("admin:aasrp_fleettype_changelist")
        self.client.login(username="admin", password="password")
        response = self.client.get(url)

        self.assertEqual(response.status_code, HTTPStatus.OK)

    def test_admin_form_saves_valid_data(self):
        """
        Test that the admin form saves valid data correctly.

        :return:
        :rtype:
        """

        url = reverse("admin:aasrp_fleettype_change", args=[self.fleet_type.pk])
        self.client.login(username="admin", password="password")
        data = {
            "name": "Updated Fleet",
            "is_enabled": False,
            "_save": "Save",  # Include the save button name to simulate form submission
        }
        response = self.client.post(url, data)

        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.fleet_type.refresh_from_db()
        self.assertEqual(self.fleet_type.name, "Updated Fleet")
        self.assertFalse(self.fleet_type.is_enabled)

    def test_activate_action_activates_selected_fleet_types(self):
        """
        Test that the activate action activates selected fleet types.

        :return:
        :rtype:
        """

        url = reverse("admin:aasrp_fleettype_changelist")
        self.client.login(username="admin", password="password")
        data = {
            "action": "activate",
            "_selected_action": [self.fleet_type.pk],
        }
        response = self.client.post(url, data, follow=True)
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.fleet_type.refresh_from_db()
        self.assertTrue(self.fleet_type.is_enabled)

    def test_deactivate_action_deactivates_selected_fleet_types(self):
        """
        Test that the deactivate action deactivates selected fleet types.

        :return:
        :rtype:
        """

        url = reverse("admin:aasrp_fleettype_changelist")
        self.client.login(username="admin", password="password")
        data = {
            "action": "deactivate",
            "_selected_action": [self.fleet_type.pk],
        }
        response = self.client.post(url, data, follow=True)
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.fleet_type.refresh_from_db()
        self.assertFalse(self.fleet_type.is_enabled)



"""
Versioned static URLs to break browser caches when changing the app version
"""

# Standard Library
import os

# Django
from django.contrib.auth.models import User
from django.template.defaulttags import register
from django.templatetags.static import static
from django.utils.safestring import mark_safe

# Alliance Auth
from allianceauth.framework.api.user import get_main_character_name_from_user
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag

# AA SRP
from aasrp import __title__, __version__
from aasrp.app_settings import debug_enabled
from aasrp.constants import PACKAGE_NAME
from aasrp.helper.static_files import calculate_integrity_hash

logger = LoggerAddTag(my_logger=get_extension_logger(__name__), prefix=__title__)


@register.simple_tag
def aasrp_static(relative_file_path: str, script_type: str = None) -> str | None:
    """
    Versioned static URL

    :param relative_file_path: The file path relative to the `{APP_NAME}/{PACKAGE_NAME}/static/{PACKAGE_NAME}` folder
    :type relative_file_path: str
    :param script_type: The script type
    :type script_type: str
    :return: Versioned static URL
    :rtype: str
    """

    logger.debug(f"Getting versioned static URL for: {relative_file_path}")

    file_type = os.path.splitext(relative_file_path)[1][1:]

    logger.debug(f"File extension: {file_type}")

    # Only support CSS and JS files
    if file_type not in ["css", "js"]:
        raise ValueError(f"Unsupported file type: {file_type}")

    static_file_path = os.path.join(PACKAGE_NAME, relative_file_path)
    static_url = static(static_file_path)

    # Integrity hash calculation only for non-debug mode
    sri_string = (
        f' integrity="{calculate_integrity_hash(relative_file_path)}" crossorigin="anonymous"'
        if not debug_enabled()
        else ""
    )

    # Versioned URL for CSS and JS files
    # Add version query parameter to break browser caches when changing the app version
    # Do not add version query parameter for libs as they are already versioned through their file path
    versioned_url = (
        static_url
        if relative_file_path.startswith("libs/")
        else static_url + "?v=" + __version__
    )

    return_value = None

    # Return the versioned URL with integrity hash for CSS
    if file_type == "css":
        return_value = mark_safe(
            f'<link rel="stylesheet" href="{versioned_url}"{sri_string}>'
        )

    # Return the versioned URL with integrity hash for JS files
    if file_type == "js":
        js_type = f' type="{script_type}"' if script_type else ""

        return_value = mark_safe(
            f'<script{js_type} src="{versioned_url}"{sri_string}></script>'
        )

    return return_value


@register.filter
def main_character_name(user: User) -> str:
    """
    Get the users main character name, or return the username if no main character

    :param user:
    :type user:
    :return:
    :rtype:
    """

    return get_main_character_name_from_user(user=user)


@register.filter
def main_character_id(user: User) -> int:
    """
    Get the users main character id, or return 1 if no main character

    :param user:
    :type user:
    :return:
    :rtype:
    """

    if user and hasattr(user.profile.main_character, "character_id"):
        return user.profile.main_character.character_id

    return 1


@register.filter
def main_character_corporation_name(user: User) -> str:
    """
    Get the users main character corporation name, or an empty string if no main character

    :param user:
    :type user:
    :return:
    :rtype:
    """

    if user and hasattr(user.profile.main_character, "corporation_name"):
        return user.profile.main_character.corporation_name

    return ""


@register.filter
def main_character_corporation_id(user: User) -> int:
    """
    Get the users main character corporation id, or 1 if no main character

    :param user:
    :type user:
    :return:
    :rtype:
    """

    if user and hasattr(user.profile.main_character, "corporation_id"):
        return user.profile.main_character.corporation_id

    return 1


@register.filter
def main_character_alliance_name(user: User) -> str:
    """
    Get the users main character alliance name, or an empty string if no main character

    :param user:
    :type user:
    :return:
    :rtype:
    """

    return getattr(user.profile.main_character, "alliance_name", "") if user else ""


@register.filter
def main_character_alliance_id(user: User) -> int:
    """
    Get the users main character alliance id, or 1 if no main character

    :param user:
    :type user:
    :return:
    :rtype:
    """

    if user is None:
        return 1

    try:
        return_value = int(user.profile.main_character.alliance_id)
    except (AttributeError, ValueError, TypeError):
        return_value = 1

    return return_value
